from django import template
from django.utils.safestring import mark_safe
from django.template.base import Node
from django.utils.translation import ugettext as _

register = template.Library()

class Header(Node):
	def render(self, context):
		username = context.get('username', None)
		realname = context.get('realname', None)
		managerpath = context.get('manager_path', None)
		if not username or username == "":
			header = """
			<div class="header_menu" id="header_menu">
				<a href=\"""" + managerpath + """+login">""" + _("Log in") + """</a> / <a href=\"""" + managerpath + """+register">""" + _("Register") + """</a>
			</div>
			"""
		elif realname == "":
			header = """
			<div class="header_menu" id="header_menu">
				<a href=\"""" + managerpath + """~""" + username + """">""" + username + """</a> / <a href=\"""" + managerpath + """+logout" style=\"font-size: 8pt;\">""" + _("Log Out") + """</a>
			</div>
			"""
		else:
			header = """
			<div class="header_menu" id="header_menu">
				<a href=\"""" + managerpath + """~""" + username + """">""" + realname + " (" + username + """)</a> / <a href=\"""" + managerpath + """+logout" style=\"font-size: 8pt;\">""" + _("Log Out") + """</a>
			</div>
			"""
		return mark_safe(header)

class HeaderNoUsername(Node):
	def render(self, context):
		managerpath = context.get('manager_path', None)
		header = """
		<div class="header_menu" id="header_menu">
			<a href=\"""" + managerpath + """+login">""" + _("Log in") + """</a> / <a href=\"""" + managerpath + """+register">""" + _("Register") + """</a>
		</div>
		"""
		return mark_safe(header)

class Footer(Node):
	def render(self, context):
		footer = """
		<div class="footer">
			<hr />
			<p>&copy; 2012 Koichi Akabe. &middot; <a href="https://code.launchpad.net/~vbkaisetsu/+junk/unaju">Unaju</a> 0.1<p>
		</div>
		"""
		return mark_safe(footer)

@register.tag
def header(parser, token):
	return Header()

@register.tag
def header_nousername(parser, token):
	return HeaderNoUsername()
	
@register.tag
def footer(parser, token):
	return Footer()
