from django.contrib.auth.models import User
from django.shortcuts import render_to_response
from django.utils.translation import ugettext as _
from manager.models import Survey
from manager.models import Sheet
from django.http import Http404, HttpResponse, HttpResponseNotModified
from django.template import RequestContext
from django.core.urlresolvers import reverse
from unaju.settings import UPLOAD_ROOT
from os import makedirs, path, remove
from email.utils import formatdate, parsedate_tz, mktime_tz
import uuid, poppler, cairo, re

def upload(request, owner):
	user = request.user
	if user.is_authenticated():
		if user.username != owner and not user.is_staff:
			raise Http403
	else:
		c = {"status": "error", "command": "stay", "message": _("Authentication is required.")}
		return render_to_response("manager/result.xml", c, mimetype="application/xml")
	if request.method == "POST" and request.FILES["pdffile"]:
		uploaddir = path.join(UPLOAD_ROOT, owner, "tmp")
		if not path.exists(uploaddir):
			makedirs(uploaddir)
		while True:
			genuuid = str(uuid.uuid4())
			filepath = path.join(uploaddir, genuuid)
			if not path.exists(filepath) and not path.exists(path.join(UPLOAD_ROOT, owner, genuuid)):
				break
		dest = open(filepath, "wb+")
		upfile = request.FILES["pdffile"]
		for chunk in upfile.chunks():
			dest.write(chunk)
		dest.close()
		try:
			pdf = poppler.document_new_from_file("file://%s" % filepath, None)
		except Exception:
			remove(filepath)
			c = {"status": "error", "command": "stay", "message": _("Attached file is not PDF.")}
			return render_to_response("manager/result.xml", c, mimetype="application/xml")
		c = {"status": "success", "command": "stay", "message": genuuid}
		return render_to_response("manager/result.xml", c, mimetype="application/xml")
	c = {"status": "error", "command": "stay", "message": _("PDF is not specified.")}
	return render_to_response("manager/result.xml", c, mimetype="application/xml")


def download_attachment(request, owner, uuid):
	user = request.user
	if user.is_authenticated():
		if user.username != owner and not user.is_staff:
			raise Http403
	filepath = path.join(UPLOAD_ROOT, owner, uuid)
	response = None
	if path.exists(filepath):
		response = HttpResponse(open(filepath, "rb").read(), mimetype="application/pdf")
	filepath = path.join(UPLOAD_ROOT, owner, "tmp", uuid)
	if path.exists(filepath):
		response = HttpResponse(open(filepath, "rb").read(), mimetype="application/pdf")
	if response:
		response["Content-Disposition"] = "filename=%s.pdf" % uuid
		return response
	raise Http404

def pdf_view(request, owner, uuid):
	user = request.user
	if user.is_authenticated():
		if user.username != owner and not user.is_staff:
			raise Http403
	uuidpattern = re.compile("([0-9a-f]+)-([0-9a-f]+)-([0-9a-f]+)-([0-9a-f]+)-([0-9a-f]+)")
	if not uuidpattern.match(uuid):
		raise Http404
	filepath = path.join(UPLOAD_ROOT, owner, uuid)
	if not path.exists(filepath):
		raise Http404
	lastmodified = int(path.getmtime(filepath))
	if "HTTP_IF_MODIFIED_SINCE" in request.META:
		modsince_tuple = parsedate_tz(request.META["HTTP_IF_MODIFIED_SINCE"])
		header_modsince = mktime_tz(modsince_tuple) if modsince_tuple else 0
		if lastmodified <= header_modsince:
			return HttpResponseNotModified()
	pdf = poppler.document_new_from_file("file://%s" % filepath, None)
	pnumstr = request.GET["page"] if "page" in request.GET else "0"
	pnum = int(pnumstr) if pnumstr.isdigit() else 0
	widthstr = request.GET["width"] if "width" in request.GET else "100"
	width = int(widthstr) if widthstr.isdigit() else 100
	if(width > 1000):
		width = 1000
	if(0 <= pnum < pdf.get_n_pages()):
		page = pdf.get_page(pnum)
		pagesize = page.get_size()
		scale = width / pagesize[0]
		height = int(pagesize[1] * scale)
		if(height > 2000):
			height = 2000;
			scale = height / pagesize[1]
			width = int(pagesize[0] * scale);
		if(width == 0 or height == 0):
			return HttpResponse("");
		surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
		cr = cairo.Context(surface)
		cr.scale(scale, scale);
		cr.save()
		page.render_for_printing(cr)
		cr.restore()
		cr.set_operator(cairo.OPERATOR_DEST_OVER)
		cr.set_source_rgb(1, 1, 1);
		cr.paint()
		response = HttpResponse(mimetype="image/png")
		surface.write_to_png(response)
		response["Last-Modified"] = formatdate(lastmodified)
		return response
	else:
		raise Http404

def pdf_xml(request, owner, uuid):
	user = request.user
	if user.is_authenticated():
		if user.username != owner and not user.is_staff:
			raise Http403
	uuidpattern = re.compile("([0-9a-f]+)-([0-9a-f]+)-([0-9a-f]+)-([0-9a-f]+)-([0-9a-f]+)")
	if not uuidpattern.match(uuid):
		raise Http404
	filepath = path.join(UPLOAD_ROOT, owner, uuid)
	if not path.exists(filepath):
		raise Http404
	lastmodified = int(path.getmtime(filepath))
	if "HTTP_IF_MODIFIED_SINCE" in request.META:
		modsince_tuple = parsedate_tz(request.META["HTTP_IF_MODIFIED_SINCE"])
		header_modsince = mktime_tz(modsince_tuple) if modsince_tuple else 0
		if lastmodified <= header_modsince:
			return HttpResponseNotModified()
	pdf = poppler.document_new_from_file("file://%s" % filepath, None)
	pagecnt = pdf.get_n_pages()
	pagedatas = []
	for pnum in range(pagecnt):
		page = pdf.get_page(pnum)
		pagesize = page.get_size()
		pagedatas.append({"width": pagesize[0], "height": pagesize[1]})
		c = {"pagedatas": pagedatas}
	response = render_to_response("manager/pdfdata.xml", c, mimetype="application/xml")
	response["Last-Modified"] = formatdate(lastmodified)
	return response
