from django.shortcuts import render_to_response
from django.core.urlresolvers import reverse
from django.core.context_processors import csrf
from django.template import RequestContext
from django.http import HttpResponseRedirect
from django.contrib.auth.models import User
from django.core.validators import email_re
from manager.models import UserConfirmation
from django.db import IntegrityError
from django.utils.translation import ugettext as _
from django.utils import timezone
from unaju.settings import URL_BASE, ADMINS
from django.core.mail import send_mail
import uuid, datetime

def register_view(request):
	if request.user.is_authenticated():
		return HttpResponseRedirect("%s~%s" % (reverse("manager"), request.user.username))
	
	errmsg = ""
	
	if "confirm" in request.GET:
		confuuid = request.GET["confirm"]
		try:
			conf = UserConfirmation.objects.filter(expire__gte=timezone.now()).get(uuid=confuuid)
		except UserConfirmation.DoesNotExist:
			c = {"manager_path": reverse("manager")}
			return render_to_response("manager/register_failed.html", c, context_instance=RequestContext(request))
		user = User.objects.get(username=conf.user)
		conf.delete()
		user.is_active = True
		user.save()
		send_mail(_("Welcome to Unaju"),
		_("""Your user id (%s) has been activated.\n""") % user.username,
		ADMINS[0][1], [user.email], fail_silently=False)
		c = {"manager_path": reverse("manager")}
		return render_to_response("manager/register_complete.html", c, context_instance=RequestContext(request))
	
	if request.method == "POST" and "user" in request.POST:
		username = request.POST["user"].strip()
		password = request.POST["pass"] if "pass" in request.POST else ""
		password_retype = request.POST["pass_retype"] if "pass_retype" in request.POST else ""
		firstname = request.POST["firstname"].strip() if "firstname" in request.POST else ""
		lastname = request.POST["lastname"].strip() if "lastname" in request.POST else ""
		email = request.POST["email"] if "email" in request.POST else ""
		if username == "":
			errmsg = _("Please type your user id")
		elif password == "":
			errmsg = _("Please type the password")
		elif password != password_retype:
			errmsg = _("Prease type same passwords in two places")
		elif len(password) < 8:
			errmsg = _("Password should be longer than 8 characters")
		elif not email_re.match(email):
			errmsg = _("Email address is not correct")
		elif len(User.objects.filter(email=email)) != 0:
			errmsg = _("This email address is already used")
		else:
			# remove unconfirmed users
			jusers = UserConfirmation.objects.filter(expire__lt=timezone.now()).all()
			for juser in jusers:
				user = User.objects.get(username=juser.user)
				user.delete()
				juser.delete()
			try:
				user = User.objects.create_user(username, email, password)
			except IntegrityError:
				errmsg = _("This user id is already used")
			else:
				user.is_staff = False
				user.first_name = firstname
				user.last_name = lastname
				user.is_active = False
				user.save()
				confuuid = uuid.uuid4().hex
				UserConfirmation(user=username, uuid=confuuid, expire=timezone.now() + datetime.timedelta(1)).save()
				c = {"manager_path": reverse("manager"), "email": email}
				send_mail(_("Please activate your id for Unaju"),
				_("""Your user id (%(id)s) is not activated.\n\nPlease access to the following address to activate your id:\n %(url)s\n""")
				 % {"id": username, "url": "%s%s+register?confirm=%s" % (URL_BASE, reverse("manager"), confuuid)},
				ADMINS[0][1], [email], fail_silently=False)
				return render_to_response("manager/register_confirm.html", c, context_instance=RequestContext(request))
			
	c = {"manager_path": reverse("manager"), "errmsg": errmsg}
	c.update(csrf(request))
	return render_to_response("manager/register.html", c, context_instance=RequestContext(request))

