from django.contrib.auth.models import User
from manager.models import Sheet, Question
from django.shortcuts import render_to_response
from django.http import HttpResponseRedirect, Http404
from django.utils.translation import ugettext as _
from django.utils import timezone
from django.template import RequestContext, Context
from django.core.urlresolvers import reverse
from django.core.context_processors import csrf
from django.template.loader import get_template
from manager.utils.survey_db import add_column, drop_column, drop_survey_table
from manager.utils.exceptions import Http403
from unaju.settings import UPLOAD_ROOT
from shutil import move
from os import path, remove
import re

def validate_sheet(title, owner, qlist):
	uuidpattern = re.compile("([0-9a-f]+)-([0-9a-f]+)-([0-9a-f]+)-([0-9a-f]+)-([0-9a-f]+)")
	if title == "":
		return False
	if len(title) > 200:
		return False
	if not qlist:
		return False
	for q in qlist:
		if not q[0].isdigit():
			return False
		if q[1] == "":
			return False
		if len(q[1]) > 400:
			return False
		if not q[2].isdigit():
			return False
		if q[2] == "0" or q[2] == "4":
			if not q[3].isdigit():
				return False
			i = int(q[3])
			if not (1 <= i <= 4000):
				return False
		elif q[2] == "1":
			if not q[3].isdigit():
				return False
			i = int(q[3])
			if not (2 <= i <= 10):
				return False
		elif q[2] == "2" or q[2] == "3":
			if len(q[3]) > 400:
				return False
		else:
			return False
		if q[4] != "":
			if not uuidpattern.match(q[4]):
				return False
			if path.exists(path.join(UPLOAD_ROOT, owner, "tmp", q[4])):
				move(path.join(UPLOAD_ROOT, owner, "tmp", q[4]), path.join(UPLOAD_ROOT, owner, q[4]))
			elif not path.exists(path.join(UPLOAD_ROOT, owner, q[4])):
				return False
			
	return True

def edit_view(request, owner="", sheet_id=""):
	username = ""
	realname = ""
	status = ""
	message = ""
	result = request.GET["result"] if "result" in request.GET else ""
	mode = request.POST["mode"] if "mode" in request.POST else ""
	if request.user.is_authenticated():
		user = request.user
		username = user.username
		realname = user.get_full_name()
		if username == owner:
			permlevel = 0
		elif user.is_staff:
			permlevel = 0
		else:
			permlevel = 3
	else:
		if result == "xml":
			c = {"status": "error", "command": "auth", "message": "%s~%s/sheet/%d/+edit/+login" % (reverse("manager"), owner, int(sheet_id))}
			return render_to_response("manager/result.xml", c, mimetype="application/xml")
		return HttpResponseRedirect("%s~%s/sheet/%d/+edit/+login" % (reverse("manager"), owner, int(sheet_id)))

	try:
		ownerdata = User.objects.get(username=owner)
	except User.DoesNotExist:
		raise Http404
	
	realownername = ownerdata.get_full_name()
	if realownername == "":
		realownername = owner

	try:
		sheet = Sheet.objects.filter(owner=owner).get(id=sheet_id)
	except Sheet.DoesNotExist:
		raise Http404
	
	if sheet.permission < permlevel:
		raise Http403
		
	if request.method == "POST":
		if mode == "update":
			title = request.POST["title"] if "title" in request.POST else ""
			item_cnt = 0
			question_list = []
			while True:
				question_id = request.POST["question_id_%d" % item_cnt] if ("question_id_%d" % item_cnt) in request.POST else ""
				question = request.POST["question_%d" % item_cnt] if ("question_%d" % item_cnt) in request.POST else ""
				ans_format = request.POST["format_%d" % item_cnt] if ("format_%d" % item_cnt) in request.POST else ""
				ans_property = request.POST["property_%d" % item_cnt] if ("property_%d" % item_cnt) in request.POST else ""
				attachment = request.POST["attachment_%d" % item_cnt] if ("attachment_%d" % item_cnt) in request.POST else ""
				if question == "":
					break
				question_list.append([question_id, question, ans_format, ans_property, attachment])
				item_cnt += 1
			
			if not validate_sheet(title, owner, question_list):
				status = "error"
				message =  _("Invalid datas are sent.")
				if result == "xml":
					c = {"status": status, "command": "stay", "message": message}
					return render_to_response("manager/result.xml", c, mimetype="application/xml")
			else:
				# update
				question_ids = [int(q[0]) for q in question_list]
				question = sheet.question_set.all()
				for q in question:
					if not q.id in question_ids:
						if q.attachment != "" and path.exists(path.join(UPLOAD_ROOT, owner, q.attachment)):
							remove(path.join(UPLOAD_ROOT, owner, q.attachment))
						for survey in sheet.survey_set.all():
							drop_column(survey, q)
						q.delete()
				for i, qdata in enumerate(question_list):
					try:
						q = sheet.question_set.get(id=qdata[0])
						q.order = i
					except Question.DoesNotExist:
						q = sheet.question_set.create(order=i, question=qdata[1], ans_format=int(qdata[2]), ans_property=qdata[3], attachment=qdata[4])
						for survey in sheet.survey_set.all():
							add_column(survey, q)
					q.save()
				sheet.title = title
				sheet.mod_date=timezone.now()
				sheet.save()

				status = "success"
				message =  _("Successfully updated.")
				if result == "xml":
					c = {"status": status, "command": "stay", "message": message}
					return render_to_response("manager/result.xml", c, mimetype="application/xml")
		elif mode == "remove":
			question = sheet.question_set.all()
			for q in question:
				if q.attachment != "" and path.exists(path.join(UPLOAD_ROOT, owner, q.attachment)):
					remove(path.join(UPLOAD_ROOT, owner, q.attachment))
				q.delete()
			for survey in sheet.survey_set.all():
				drop_survey_table(survey)
				survey.delete()
			sheet.delete()
			
			request.session["removed"] = True
			if result == "xml":
				c = {"status": "success", "command": "jump", "message": "%s~%s" % (reverse("manager"), owner)}
				return render_to_response("manager/result.xml", c, mimetype="application/xml")
			return HttpResponseRedirect("%s~%s" % (reverse("manager"), owner))
		
	if "created" in request.session and request.session.pop("created"):
		status = "success"
		message = _("Successfully created.")
	c = {"username": username, "realname": realname, "owner": owner, "realownername": realownername, "sheetid": sheet_id, "status": status, "message": message, "title": _("Edit sheet"), "permlevel": permlevel, "manager_path": reverse("manager")}
	c.update(csrf(request))
	return render_to_response("manager/sheetedit.html", c, context_instance=RequestContext(request))

def add_view(request, owner=""):
	username = ""
	realname = ""
	status = ""
	message = ""
	result = request.GET["result"] if "result" in request.GET else ""
	mode = request.POST["mode"] if "mode" in request.POST else ""
	if request.user.is_authenticated():
		user = request.user
		username = user.username
		realname = user.get_full_name()
		if username != owner and not user.is_staff:
			raise Http403
	else:
		if result == "xml":
			c = {"status": "error", "command": "auth", "message": "/manager/~%s/sheet/+add/+login" % owner}
			return render_to_response("manager/result.xml", c, mimetype="application/xml")
		return HttpResponseRedirect("%s~%s/sheet/+add/+login" % (reverse("manager"), owner))

	try:
		ownerdata = User.objects.get(username=owner)
	except User.DoesNotExist:
		if result == "xml":
			c = {"status": "error", "command": "stay", "message": _("User does not exist.")}
			return render_to_response("manager/result.xml", c, mimetype="application/xml")
		raise Http404
	
	realownername = ownerdata.get_full_name()
	if realownername == "":
		realownername = owner

	if request.method == "POST":
		if mode == "update":
			title = request.POST["title"] if "title" in request.POST else ""
			item_cnt = 0
			question_list = []
			while True:
				question = request.POST["question_%d" % item_cnt] if ("question_%d" % item_cnt) in request.POST else ""
				ans_format = request.POST["format_%d" % item_cnt] if ("format_%d" % item_cnt) in request.POST else ""
				ans_property = request.POST["property_%d" % item_cnt] if ("property_%d" % item_cnt) in request.POST else ""
				attachment = request.POST["attachment_%d" % item_cnt] if ("attachment_%d" % item_cnt) in request.POST else ""
				if question == "":
					break
				question_list.append(["0", question, ans_format, ans_property, attachment])
				item_cnt += 1
			
			if not validate_sheet(title, owner, question_list):
				status = "error"
				message =  _("Invalid datas are sent.")
				if result == "xml":
					c = {"status": status, "command": "stay", "message": message}
					return render_to_response("manager/result.xml", c, mimetype="application/xml")
			else:
				sheet = Sheet(title=title, create_date=timezone.now(), mod_date=timezone.now(), owner=owner, permission=0)
				sheet.save()
				for i, question in enumerate(question_list):
					q = sheet.question_set.create(order=i, question=question[1], ans_format=question[2], ans_property=question[3], attachment=question[4])
					q.save()
				
				request.session["created"] = True
				if result == "xml":
					c = {"status": "success", "command": "jump", "message": "%s~%s/sheet/%d/+edit" % (reverse("manager"), owner, sheet.id)}
					return render_to_response("manager/result.xml", c, mimetype="application/xml")
				return HttpResponseRedirect("%s~%s/sheet/%d/+edit" % (reverse("manager"), owner, sheet.id))
		
	c = {"username": username, "realname": realname, "owner": owner, "sheetid": "", "realownername": realownername, "status": status, "message": message, "title": _("Add sheet"), "manager_path": reverse("manager")}
	return render_to_response("manager/sheetedit.html", c, context_instance=RequestContext(request))
