from django.contrib.auth.models import User
from manager.models import Survey
from django.shortcuts import render_to_response
from django.http import HttpResponseRedirect, HttpResponse, Http404
from manager.utils import survey_db
from django.template import RequestContext
from django.core.urlresolvers import reverse
from manager.utils.exceptions import Http403
from django.utils.translation import ugettext as _

def survey_view(request, owner, survey_id):
	status = ""
	message = ""
	if request.user.is_authenticated():
		user = request.user
		username = user.username
		realname = user.get_full_name()
		if username == owner:
			permlevel = 0
		elif user.is_staff:
			permlevel = 0
		else:
			permlevel = 1
	else:
		permlevel = 2
		username = ""
		realname = ""

	try:
		ownerdata = User.objects.get(username=owner)
	except User.DoesNotExist:
		raise Http404

	realownername = ownerdata.get_full_name()
	if realownername == "":
		realownername = owner
	
	try:
		survey = Survey.objects.filter(owner=owner).get(id=survey_id)
	except Survey.DoesNotExist:
		raise Http404

	if survey.permission < permlevel:
		return HttpResponseRedirect("./%d/+login" % int(survey_id))

	if "started" in request.session and request.session.pop("started"):
		status = "success"
		message = _("Successfully started.")
	if "stopped" in request.session and request.session.pop("stopped"):
		status = "success"
		message = _("Successfully stopped.")
	if "reset" in request.session and request.session.pop("reset"):
		status = "success"
		message = _("Successfully reset.")
	c = {"username": username, "realname": realname, "owner": owner, "realownername": realownername, "surveyid": survey_id, "sheetid": survey.sheet.id, "permlevel": permlevel, "status": status, "message": message, "manager_path": reverse("manager")}
	return render_to_response("manager/survey.html", c, context_instance=RequestContext(request))

def xml_view(request, owner="", survey_id=""):
	if request.user.is_authenticated():
		user = request.user
		username = user.username
		realname = user.get_full_name()
		if username == owner:
			permlevel = 0
		elif user.is_staff:
			permlevel = 0
		else:
			permlevel = 1
	else:
		permlevel = 2

	try:
		ownerdata = User.objects.get(username=owner)
	except User.DoesNotExist:
		raise Http404
	
	try:
		survey = Survey.objects.filter(owner=owner).get(id=survey_id)
	except Survey.DoesNotExist:
		raise Http404

	if survey.permission < permlevel:
		raise Http403

	c = {"title": survey.title, "status": survey.status}
	return render_to_response("manager/survey.xml", c, mimetype="application/xml")

def resultxml_view(request, owner="", survey_id=""):
	if request.user.is_authenticated():
		user = request.user
		username = user.username
		realname = user.get_full_name()
		if username == owner:
			permlevel = 0
		elif user.is_staff:
			permlevel = 0
		else:
			permlevel = 1
	else:
		permlevel = 2

	try:
		ownerdata = User.objects.get(username=owner)
	except User.DoesNotExist:
		raise Http404
	
	try:
		survey = Survey.objects.filter(owner=owner).get(id=survey_id)
	except Survey.DoesNotExist:
		raise Http404

	if survey.permission < permlevel:
		raise Http403
		
	start_str = request.GET["start"] if "start" in request.GET else ""
	start = int(start_str) if start_str.isdigit() else 0
	results = survey_db.select_from_survey_table(survey, start)

	c = {"title": survey.title, "results": results}
	return render_to_response("manager/surveyresult.xml", c, mimetype="application/xml")

def download_csv(request, owner="", survey_id=""):
	if request.user.is_authenticated():
		user = request.user
		username = user.username
		realname = user.get_full_name()
		if username == owner:
			permlevel = 0
		elif user.is_staff:
			permlevel = 0
		else:
			permlevel = 1
	else:
		permlevel = 2

	try:
		ownerdata = User.objects.get(username=owner)
	except User.DoesNotExist:
		raise Http404
	
	try:
		survey = Survey.objects.filter(owner=owner).get(id=survey_id)
	except Survey.DoesNotExist:
		raise Http404

	if survey.permission < permlevel:
		raise Http403
		
	results = survey_db.select_from_survey_table(survey, 0)
	
	csv = ""
	ans_formats = {}
	questions = survey.sheet.question_set.order_by("order")
	for i, q in enumerate(questions):
		ans_formats[q.id] = {"order": i, "format": q.ans_format, "data": ""}
		csv += "# %s %d\n" % (_("Column"), i)
		csv += "# %s: %s\n" % (_("Question"), "\n# ".join(q.question.split("\n")))
		if q.ans_format == 0:
			csv += "# %s: %s\n" % (_("Format"), _("Text"))
			csv += "# %s: %s\n#\n" % (_("Maximum characters"), q.ans_property)
		elif q.ans_format == 1:
			csv += "# %s: %s\n" % (_("Format"), _("Ranking"))
			csv += "# %s: %s\n#\n" % (_("Maximum value"), q.ans_property)
		elif q.ans_format == 2:
			csv += "# %s: %s\n" % (_("Format"), _("Choices (single selection)"))
			csv += "# %s: %s\n#\n" % (_("Choices"), q.ans_property)
		elif q.ans_format == 3:
			csv += "# %s: %s\n" % (_("Format"), _("Choices (multiple selection)"))
			csv += "# %s: %s\n#\n" % (_("Choices"), q.ans_property)
		elif q.ans_format == 4:
			csv += "# %s: %s\n" % (_("Format"), _("Stream"))
			csv += "# %s: %s\n#\n" % (_("Maximum characters"), q.ans_property)
	for result in results:
		result_sorted = []
		for item in result:
			if ans_formats[item["id"]]["format"] == 0 or ans_formats[item["id"]]["format"] == 3 or ans_formats[item["id"]]["format"] == 4:
				r = {"order": ans_formats[item["id"]]["order"], "data": "\"" + item["data"].replace("\\", "\\\\").replace("\"", "\\\"") + "\""}
			elif ans_formats[item["id"]]["format"] == 1 or ans_formats[item["id"]]["format"] == 2:
				r = {"order": ans_formats[item["id"]]["order"], "data": str(item["data"]) if item["data"] != -1 else ""}
			else:
				return
			result_sorted.append(r)
		result_sorted.sort(cmp=lambda x,y: cmp(x["order"], y["order"]))
		csv += ",".join([r["data"] for r in result_sorted])
		csv += "\n"
	response = HttpResponse(csv, content_type="text/csv")
	response["Content-Disposition"] = "attachment; filename=\"result.csv\""
	return response
