from django.contrib.auth.models import User
from manager.models import Sheet, Survey
from manager.utils import survey_db
from django.shortcuts import render_to_response
from django.http import HttpResponseRedirect, Http404
from django.utils.translation import ugettext as _
from django.utils import timezone
from django.template import RequestContext
from django.core.urlresolvers import reverse
import re
from manager.utils.exceptions import Http403

def validate_survey(title, m):
	if title == "":
		return False
	if len(title) > 200:
		return False
	if not m:
		return False
	return True

def edit_view(request, owner, survey_id):
	username = ""
	realname = ""
	status = ""
	message = ""
	result = request.GET["result"] if "result" in request.GET else ""
	mode = request.POST["mode"] if "mode" in request.POST else ""
	if request.user.is_authenticated():
		user = request.user
		username = user.username
		realname = user.get_full_name()
		if username == owner:
			permlevel = 0
		elif user.is_staff:
			permlevel = 0
		else:
			permlevel = 3
	else:
		if result == "xml":
			c = {"status": "error", "command": "auth", "message": "%s~%s/survey/%d/+edit/+login" % (reverse("manager"), owner, int(survey_id))}
			return render_to_response("manager/result.xml", c, mimetype="application/xml")
		return HttpResponseRedirect("%s~%s/survey/%d/+edit/+login" % (reverse("manager"), owner, int(survey_id)))

	try:
		ownerdata = User.objects.get(username=owner)
	except User.DoesNotExist:
		raise Http404
	
	realownername = ownerdata.get_full_name()
	if realownername == "":
		realownername = owner

	try:
		survey = Survey.objects.filter(owner=owner).get(id=survey_id)
	except Survey.DoesNotExist:
		raise Http404
	
	if survey.permission < permlevel:
		raise Http403
		
	if request.method == "POST":
		if mode == "update":
			title = request.POST["title"] if "title" in request.POST else ""
			if not validate_survey(title, m):
				status = "error"
				message =  _("Invalid datas are sent.")
				if result == "xml":
					c = {"status": status, "command": "stay", "message": message}
					return render_to_response("manager/result.xml", c, mimetype="application/xml")
			
			survey.title = title
			survey.mod_date=timezone.now()
			survey.save()

			status = "success"
			message =  _("Successfully updated.")
			if result == "xml":
				c = {"status": status, "command": "stay", "message": message}
				return render_to_response("manager/result.xml", c, mimetype="application/xml")

		elif mode == "remove":
			sheet_id = survey.sheet.id
			survey_db.drop_survey_table(survey)
			survey.delete()

			request.session["removed"] = True
			if result == "xml":
				c = {"status": "success", "command": "jump", "message": "%s~%s/sheet/%d/list" % (reverse("manager"), owner, sheet_id)}
				return render_to_response("manager/result.xml", c, mimetype="application/xml")
			return HttpResponseRedirect("%s~%s/sheet/%d/list" % (reverse("manager"), owner, sheet_id))

		elif mode == "start":
			if survey.pub_date == None:
				survey.pub_date = timezone.now()
			survey.status = 1
			survey.save()
			
			if result == "xml":
				c = {"status": "success", "command": "stay", "message": _("Successfully started.")}
				return render_to_response("manager/result.xml", c, mimetype="application/xml")
			request.session["started"] = True
			return HttpResponseRedirect("../%s" % survey_id)

		elif mode == "stop":
			survey.status = 2
			survey.save()
			
			if result == "xml":
				c = {"status": "success", "command": "stay", "message": _("Successfully stopped.")}
				return render_to_response("manager/result.xml", c, mimetype="application/xml")
			request.session["stopped"] = True
			return HttpResponseRedirect("../%s" % survey_id)

		elif mode == "reset":
			survey.pub_date = timezone.now()
			survey.save()
			status = survey.status
			survey_db.drop_survey_table(survey)
			survey_db.create_survey_table(survey)
			
			if result == "xml":
				c = {"status": "success", "command": "stay", "message": _("Successfully reset.")}
				return render_to_response("manager/result.xml", c, mimetype="application/xml")
			request.session["reset"] = True
			return HttpResponseRedirect("../%s" % survey_id)

	if "created" in request.session and request.session.pop("created"):
		status = "success"
		message = _("Successfully created.")
	c = {"username": username, "realname": realname, "owner": owner, "realownername": realownername, "surveyid": survey_id, "sheetid": survey.sheet.id, "status": status, "message": message, "title": _("Edit survey"), "permlevel": permlevel, "manager_path": reverse("manager")}
	return render_to_response("manager/surveyedit.html", c, context_instance=RequestContext(request))

def add_view(request, owner, sheet_id):
	username = ""
	realname = ""
	status = ""
	message = ""
	result = request.GET["result"] if "result" in request.GET else ""
	mode = request.POST["mode"] if "mode" in request.POST else ""
	if request.user.is_authenticated():
		user = request.user
		username = user.username
		realname = user.get_full_name()
		if username != owner and not user.is_staff:
			if result == "xml":
				c = {"status": "error", "command": "stay", "message": _("You don't have a permission.")}
				return render_to_response("manager/result.xml", c, mimetype="application/xml")
			c = {"username": username, "realname": realname}
			return render_to_response("manager/permission_denied.html", c, context_instance=RequestContext(request))
	else:
		if result == "xml":
			c = {"status": "error", "command": "auth", "message": "./%s/+login" % sheet_id}
			return render_to_response("manager/result.xml", c, mimetype="application/xml")
		return HttpResponseRedirect("./%s/+login" % sheet_id)

	try:
		ownerdata = User.objects.get(username=owner)
	except User.DoesNotExist:
		raise Http404

	realownername = ownerdata.get_full_name()
	if realownername == "":
		realownername = owner

	if request.method == "POST":
		if mode == "update":
			title = request.POST["title"] if "title" in request.POST else ""
			try:
				sheet = Sheet.objects.filter(owner=owner).get(id=sheet_id)
			except Survey.DoesNotExist:
				status = "error"
				message =  _("Invalid datas are sent.")
				if result == "xml":
					c = {"status": status, "command": "stay", "message": message}
					return render_to_response("manager/result.xml", c, mimetype="application/xml")
			else:
				if username == owner:
					permlevel = 0
				elif user.is_staff:
					permlevel = 0
				else:
					permlevel = 1
				if sheet.permission < permlevel:
					status = "error"
					message =  _("Invalid datas are sent.")
					if result == "xml":
						c = {"status": status, "command": "stay", "message": message}
						return render_to_response("manager/result.xml", c, mimetype="application/xml")
			
			if status != "error":
				survey = sheet.survey_set.create(title=title, create_date=timezone.now(), mod_date=timezone.now(), owner=owner, permission=0, status=0)
				survey.save()
				
				survey_db.create_survey_table(survey)
				
				request.session["created"] = True
				if result == "xml":
					c = {"status": "success", "command": "jump", "message": "%s~%s/survey/%d/+edit" % (reverse("manager"), owner, survey.id)}
					return render_to_response("manager/result.xml", c, mimetype="application/xml")
				return HttpResponseRedirect("%s~%s/survey/%d/+edit" % (reverse("manager"), owner, survey.id))

	c = {"username": username, "realname": realname, "owner": owner, "realownername": realownername, "sheetid": sheet_id, "status": status, "message": message, "title": _("Add survey"), "manager_path": reverse("manager")}
	return render_to_response("manager/surveyedit.html", c, context_instance=RequestContext(request))
	
