from django.contrib.auth.models import User
from django.shortcuts import render_to_response
from django.utils.translation import ugettext as _
from manager.models import Survey
from manager.models import Sheet
from django.http import Http404
from django.template import RequestContext
from django.core.urlresolvers import reverse

def xml_view(request, owner, sheet_id):
	try:
		ownerdata = User.objects.get(username=owner)
	except User.DoesNotExist:
		raise Http404
	
	user = request.user
	if user.is_authenticated():
		if user.username == owner:
			permlevel = 0
		elif user.is_staff:
			permlevel = 0
		else:
			permlevel = 1
	else:
		permlevel = 3

	start = request.GET["start"] if "start" in request.GET else "0"
	start = int(start) if start.isdigit() else 0
	length = request.GET["length"] if "length" in request.GET else "10"
	length = int(length) if length.isdigit() else 10
	if length > 30:
		length = 30
	# owner = owner, permission >= permlevel
	
	try:
		sheet = Sheet.objects.filter(owner=owner).get(id=sheet_id)
	except Sheet.DoesNotExist:
		raise Http404
	surveys = sheet.survey_set.filter(permission__gte=permlevel).order_by("-mod_date")[start:start+length]
	
	c = {"surveys": surveys}
	return render_to_response("manager/surveylist.xml", c, mimetype="application/xml")

def surveylist_view(request, owner, sheet_id):
	username = ""
	realname = ""
	status = ""
	message = ""
	
	user = request.user
	if user.is_authenticated():
		username = user.username
		realname = user.get_full_name()
		if username == owner:
			permlevel = 0
		elif user.is_staff:
			permlevel = 0
		else:
			permlevel = 3
	else:
		permlevel = 3
	try:
		ownerdata = User.objects.get(username=owner)
	except User.DoesNotExist:
		raise Http404
	try:
		sheet = Sheet.objects.filter(owner=owner).get(id=sheet_id)
	except Sheet.DoesNotExist:
		raise Http404
	realownername = ownerdata.get_full_name()
	if realownername == "":
		realownername = owner
		
	if "removed" in request.session and request.session.pop("removed"):
		status = "success"
		message = _("Successfully removed.")
	c = {"username": username, "realname": realname, "realownername": realownername, "owner": owner, "sheetid": sheet_id, "title": sheet.title, "status": status, "message": message, "manager_path": reverse("manager"), "permlevel": permlevel}
	return render_to_response("manager/surveylist.html", c, context_instance=RequestContext(request))
