from django.contrib.auth.models import User
from django.shortcuts import render_to_response
from manager.models import Survey
from django.utils.translation import ugettext as _
from django.http import Http404
from django.core.context_processors import csrf
from django.template import RequestContext
from django.core.urlresolvers import reverse
from polls.utils import survey_db, useragent
from django.views.decorators.csrf import csrf_exempt
from django.middleware.csrf import CsrfViewMiddleware

def csrf_check(request):
	return CsrfViewMiddleware().process_view(request, csrf_check, None, None) == None

@csrf_exempt
def poll_view(request, survey_id=""):
	username = ""
	realname = ""
	errmsg = ""
	if request.user.is_authenticated():
		username = request.user.username
		realname = request.user.get_full_name()
	try:
		survey = Survey.objects.get(id=survey_id)
	except Survey.DoesNotExist:
		raise Http404
		
	title = survey.title
	
	classical_mobile = useragent.is_classical_mobile(request.META["HTTP_USER_AGENT"])
	
	if survey.status != 1:
		c = {"title": title, "manager_path": reverse("manager")}
		if classical_mobile:
			return render_to_response("polls/classic/closed.html", c, context_instance=RequestContext(request))
		else:
			return render_to_response("polls/closed.html", c, context_instance=RequestContext(request))

	sheet = survey.sheet
	stream_only = False
	datanull = True
	if ("posted_%s" % survey_id) in request.session and request.session["posted_%s" % survey_id] == survey.pub_date:
		for q in survey.sheet.question_set.all():
			if q.ans_format == 4: # Stream
				stream_only = True
				break
		if not stream_only:
			c = {"title": title, "manager_path": reverse("manager")}
			if classical_mobile:
				return render_to_response("polls/classic/already.html", c, context_instance=RequestContext(request))
			else:
				return render_to_response("polls/already.html", c, context_instance=RequestContext(request))
	
	if request.method == "POST":
		datas = []
		error = False
		has_stream = False
		for q in survey.sheet.question_set.all():
			if not ("q_%d" % q.id) in request.POST:
				if not stream_only or stream_only and q.ans_format == 4:
					error = True
					break
			if q.ans_format == 0:
				data = request.POST["q_%d" % q.id] if not stream_only else ""
				if data != "":
					datanull = False
				if len(data) > int(q.ans_property):
					error = True
					break
				datas.append(data)
			elif q.ans_format == 1:
				data = request.POST["q_%d" % q.id] if not stream_only else "-1"
				if not stream_only:
					datanull = False
					if not (data.isdigit() and 1 <= int(data) <= int(q.ans_property)):
						error = True
						break
				datas.append(int(data))
			elif q.ans_format == 2:
				data = request.POST["q_%d" % q.id] if not stream_only else "-1"
				if not stream_only:
					datanull = False
					if not (data.isdigit() and 0 <= int(data) < len(q.ans_property.split(","))):
						error = True
						break
				datas.append(int(data))
			elif q.ans_format == 3:
				data = request.POST.getlist("q_%d" % q.id) if not stream_only else ""
				if not stream_only:
					datanull = False
					for d in data:
						if not (d.isdigit() and 0 <= int(d) < len(q.ans_property.split(","))):
							error = True
							break
				strdata = ",".join(data)
				datas.append(strdata)
			elif q.ans_format == 4:
				has_stream = True
				data = request.POST["q_%d" % q.id]
				if data != "":
					datanull = False
				if len(data) > int(q.ans_property):
					error = True
					break
				datas.append(data)
		if not datanull:
			if not classical_mobile and not csrf_check(request):
				errmsg = _("Please accept cookie to protect against CSRF")
			elif not error:
				survey_db.addnew_to_survey_table(survey, datas)
				request.session["posted_%s" % survey_id] = survey.pub_date
				c = {"title": title, "id": survey.id, "has_stream": has_stream, "manager_path": reverse("manager")}
				if classical_mobile:
					return render_to_response("polls/classic/thanks.html", c, context_instance=RequestContext(request))
				else:
					return render_to_response("polls/thanks.html", c, context_instance=RequestContext(request))
			else:
				errmsg = _("Invalid datas are sent")
	
	sheet = []
	qnumber = 1
	for q in survey.sheet.question_set.all():
		if stream_only and q.ans_format != 4:
			qnumber += 1
			continue
		if q.ans_format == 0:
			text = request.POST["q_%d" % q.id][:int(q.ans_property)] if "q_%d" % q.id in request.POST else ""
			ans_property = {"maxlen": q.ans_property, "text": text}
		elif q.ans_format == 1:
			value_str = request.POST["q_%d" % q.id] if "q_%d" % q.id in request.POST else ""
			value = int(value_str) if value_str.isdigit() else -1
			ans_property = [{"val": i, "text": i, "selected": i == value} for i in range(1, int(q.ans_property) + 1)]
		elif q.ans_format == 2:
			value_str = request.POST["q_%d" % q.id] if "q_%d" % q.id in request.POST else ""
			value = int(value_str) if value_str.isdigit() else -1
			ans_property = [{"val": i, "text": text, "selected": i == value} for i, text in enumerate(q.ans_property.split(","))]
		elif q.ans_format == 3:
			value_str = request.POST.getlist("q_%d" % q.id) if "q_%d" % q.id in request.POST else []
			value = [int(v) for v in value_str if v.isdigit()]
			ans_property = [{"val": i, "text": text, "selected": i in value} for i, text in enumerate(q.ans_property.split(","))]
		elif q.ans_format == 4:
			text = request.POST["q_%d" % q.id][:int(q.ans_property)] if "q_%d" % q.id in request.POST else ""
			ans_property = {"maxlen": q.ans_property, "text": text}
		question = {"qnumber": qnumber, "id": q.id, "question": q.question, "format": q.ans_format, "property": ans_property}
		sheet.append(question)
		qnumber += 1

	c = {"title": title, "id": survey.id, "sheet": sheet, "errmsg": errmsg, "manager_path": reverse("manager")}
	c.update(csrf(request))
	
	# check classical mobile phone
	user_agent = request.META["HTTP_USER_AGENT"]
	if classical_mobile:
		return render_to_response("polls/classic/poll.html", c, context_instance=RequestContext(request))
	else:
		return render_to_response("polls/poll.html", c, context_instance=RequestContext(request))
